/* --------------------------------------------------------- */
/*	2state sram test										 */
/* ========================================================= */
/*	(C)2004 t.hara											 */
/* --------------------------------------------------------- */

#include <stdio.h>

#define	LCD_DATA	0xFFFFD3
#define	ASTCR		0xFFFFED

typedef struct {
	unsigned char b7:1;
	unsigned char b6:1;
	unsigned char b5:1;
	unsigned char b4:1;
	unsigned char b3:1;
	unsigned char b2:1;
	unsigned char b1:1;
	unsigned char b0:1;
} BIT;

#define	POKE8( Adr, Data )	( *(volatile unsigned char*)(Adr) = (Data) )

/* --------------------------------------------------------- */
static void WaitLcd( void ) {
	volatile int i;

	for( i = 0; i < 170; i++ );
}

/* --------------------------------------------------------- */
static void LcdPutc( char c ) {

	POKE8( LCD_DATA, ((c >> 4) & 0x0F) | 0x30 );
	WaitLcd();
	POKE8( LCD_DATA, ((c >> 4) & 0x0F) | 0x10 );
	WaitLcd();
	POKE8( LCD_DATA, (c & 0x0F) | 0x30 );
	WaitLcd();
	POKE8( LCD_DATA, (c & 0x0F) | 0x10 );
	WaitLcd();
}

/* --------------------------------------------------------- */
static void LcdCommand( char c ) {

	POKE8( LCD_DATA, ((c >> 4) & 0x0F) | 0x20 );
	WaitLcd();
	POKE8( LCD_DATA, ((c >> 4) & 0x0F) | 0x00 );
	WaitLcd();
	POKE8( LCD_DATA, (c & 0x0F) | 0x20 );
	WaitLcd();
	POKE8( LCD_DATA, (c & 0x0F) | 0x00 );
	WaitLcd();
}

/* --------------------------------------------------------- */
static void LcdPuts( const char *pStr ) {

	while( *pStr ) {
		LcdPutc( *pStr );
		++pStr;
	}
}

/* --------------------------------------------------------- */
static void Test( void ) {
	char sLineBuffer[ 22 ];
	int i;
	volatile int j;

	for( i = 0; i < 100; i++ ) {
		LcdCommand( 0x40 | 0x80 );
		sprintf( sLineBuffer, "Count[%d]    ", i );
		LcdPuts( sLineBuffer );
		for( j = 0; j < 10000; j++ ) {
		}
	}
}

/* --------------------------------------------------------- */
static void SramSpeed2State( void ) {

	((volatile BIT *)ASTCR)->b1 = 0;
}

/* --------------------------------------------------------- */
static void SramSpeed3State( void ) {

	((volatile BIT *)ASTCR)->b1 = 1;
}

/* --------------------------------------------------------- */
void main( void ) {

	while( 1 ) {
		LcdCommand( 0x00 | 0x80 );
		LcdPuts( "3state SRAM Speed" );
		Test();
		SramSpeed2State();
		LcdCommand( 0x00 | 0x80 );
		LcdPuts( "2state SRAM Speed" );
		Test();
		SramSpeed3State();
	}
}
